
#include <Wire.h> // Library for I2C communication
#include <LiquidCrystal_I2C.h> // Library for LCD
#include "Servo.h"
#include "HX711.h"

// LCD VARIABLES
// Wiring: SDA pin is connected to A4 and SCL pin to A5.
// Connect to LCD via I2C, default address 0x27 (A0-A2 not jumpered)
LiquidCrystal_I2C lcd = LiquidCrystal_I2C(0x27, 16, 2); // Change to (0x27,20,4) for 20x4 LCD.


// CONNECTIONS

int Moisture_Sensor_Pin = A1;
int Buzzer_Pin = 2;
int Builtin_Led = 13;


//Moisture Sensor Variables

int Moisture_Sensor = 0;
int Moisture_Sensor_ADC = 0;


// Servo Motor Variables

Servo Myservo;
int pos;
int Treat_Output_STS  = 0;
int LCD_State_Counter = 0;


// Load Cell Variables

#define calibration_factor -48500.0 //This value is obtained using the SparkFun_HX711_Calibration sketch

#define DOUT  4
#define CLK  5

HX711 scale;

// Logical Operations Variables

float Weight_Sensor = 0;
int Message = 0;
int Message_Servo = 0;
float Weight_Setpoint = 0;
int LCD_State = 0;

//Timings

unsigned long previousMillis_1 = 0;        // will store last time LED was updated
unsigned long previousMillis_2 = 0;        // will store last time LED was updated


void setup() {
  Serial.begin(115200);
  Serial.println("Ardunio Nano Setup Stage");
  LCD_Setup_Stage();

  pinMode(Builtin_Led, OUTPUT);
  Setup_Moisture_Sensor();
  ServoMotor_Setup_Stage();
  Load_Cell_Setup_Stage();
  Buzzer_Setup_Stage();




}
void loop() {

  Waiting();
  Communicating_With_ESP32();

  unsigned long currentMillis_1 = millis();

  if (currentMillis_1 - previousMillis_1 >= 500) {
    // save the last time you blinked the LED
    previousMillis_1 = currentMillis_1;
    LCD_Loop_Stage();
  }

  unsigned long currentMillis_2 = millis();

  if (currentMillis_2 - previousMillis_2 >= 1000) {
    // save the last time you blinked the LED
    previousMillis_2 = currentMillis_2;
    Logical_Operations();


  }

}


void Waiting() {

  Loop_Moisture_Sensor();
  Load_Cell_Loop_Stage();

  digitalWrite(Builtin_Led, LOW);
  delay(10);
  digitalWrite(Builtin_Led, HIGH);
  delay(10);
  digitalWrite(Builtin_Led, LOW);
  delay(10);



}

void Logical_Operations() {

  Serial.println("*************************** Logical Operations *************************");

  if (Message == 0) {
    Serial.println("*************************** AUTO MODE *************************");
    Serial.print("Weight_Sensor = ");
    Serial.println(Weight_Sensor);
    Serial.print("Weight_Setpoint = ");
    Serial.println(Weight_Setpoint);
    Serial.print("Moisture_Sensor = ");
    Serial.println(Moisture_Sensor);


    if (LCD_State_Counter >= 3) {
      LCD_State_Counter = 1;
    } else {
      LCD_State_Counter = LCD_State_Counter + 1;
    }


    if (LCD_State_Counter == 1) {
      LCD_State = 2;
    } else if (LCD_State_Counter == 2) {
      LCD_State = 1;
    }



    if (Weight_Sensor > Weight_Setpoint) {
      Serial.println("Weight Greater than Setpoint! Animal has Pooped!");
      Serial.println("Opening Treats");

      digitalWrite(Buzzer_Pin, HIGH);

      if (Treat_Output_STS == 0) {
        for (pos = 0; pos <= 90; pos++) {
          Myservo.write(pos);
          delay(5);
        }

        for (pos = 90; pos >= 0; pos--) {
          Myservo.write(pos);
          delay(5);
        }
        Treat_Output_STS = 1;
        digitalWrite(Buzzer_Pin, LOW);
      }


    } else if (Weight_Sensor <= Weight_Setpoint) {
      Serial.println("Weight Lower than Setpoint! Waiting for Animal Poop!");
      Treat_Output_STS = 0;
    }
  } else if (Message == 1) {
    Serial.println("*************************** MANUAL SERVO OFF *************************");
    Myservo.write(0);
  } else if (Message == 2) {
    Serial.println("*************************** MANUAL SERVO ON *************************");
    Myservo.write(90);
  }




}

void Communicating_With_ESP32() {

  Serial.print("\b");
  Serial.print(Weight_Sensor);
  Serial.print("\e");
  Serial.print(Moisture_Sensor);
  Serial.print("\f");
  Serial.print(Treat_Output_STS);
  Serial.print("\v");
  Serial.println("...");
  Serial.flush();
  Waiting();


  String reply1;
  String reply2;
  String reply3;

  reply1 = Serial.readStringUntil('\r');
  reply2 = Serial.readStringUntil('\t');
  reply3 = Serial.readStringUntil('\a');

  Serial.print("From ESP32 Controller: "); Serial.println(reply1);
  Serial.print("From ESP32 Controller: "); Serial.println(reply2);

  Message = reply2.toInt();

  Weight_Setpoint = reply3.toFloat();

  Serial.print("Message = ");
  Serial.println(Message);

  Serial.print("Weight_Setpoint = ");
  Serial.println(Message);

}


void Load_Cell_Setup_Stage() {
  Serial.println("HX711 scale demo");

  scale.begin(DOUT, CLK);
  scale.set_scale(calibration_factor); //This value is obtained by using the SparkFun_HX711_Calibration sketch
  scale.tare(); //Assuming there is no weight on the scale at start up, reset the scale to 0

  Serial.println("Readings:");
}

void Load_Cell_Loop_Stage() {
  Serial.print("Reading: ");
  Serial.print((scale.get_units()) * -1, 1); //scale.get_units() returns a float
  Serial.print(" lbs"); //You can change this to kg but you'll need to refactor the calibration_factor
  Serial.println();
  Weight_Sensor = (scale.get_units()) * -1;

}



void LCD_Setup_Stage() {
  // Initiate the LCD:
  lcd.init();
  lcd.backlight();
}

void LCD_Loop_Stage() {

  if (LCD_State == 1) {
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Weight= ");
    lcd.print(Weight_Sensor);
    lcd.print("lbs");
    lcd.setCursor(0, 1); //Set the cursor on the third column and the second row (counting starts at 0!).
    lcd.print("Setpoint= ");
    lcd.print(Weight_Setpoint);
    lcd.print("lbs");
  } else if (LCD_State == 2) {
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Moisture= ");
    lcd.print(Moisture_Sensor);
    lcd.print("%");
    lcd.setCursor(0, 1); //Set the cursor on the third column and the second row (counting starts at 0!).
    lcd.print("PET is PEEING!");
  }

}


void Setup_Moisture_Sensor () {
  pinMode(Moisture_Sensor_Pin, INPUT);
}

void Loop_Moisture_Sensor() {

  Moisture_Sensor_ADC = analogRead(Moisture_Sensor_Pin);
  Serial.print("Moisture_Sensor_ADC = ");
  Serial.println(Moisture_Sensor_ADC);
  Moisture_Sensor = map(Moisture_Sensor_ADC , 200 , 550 , 100 , 0);
  Serial.print("Moisture_Sensor = ");
  Serial.println(Moisture_Sensor);
}


void ServoMotor_Setup_Stage() {
  Myservo.attach(3);
}

void Buzzer_Setup_Stage() {
  pinMode(Buzzer_Pin, OUTPUT);
  digitalWrite(Buzzer_Pin, LOW);
}
